@@ -14,7 +14,7 @@
 
 //! Definitions of built-in functions.
 
-use {dmntk_common::errors::DmntkError, std::str::FromStr};
+use {dmntk_common::DmntkError, std::str::FromStr};
 
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub enum Bif {
@@ -197,7 +197,7 @@ pub fn is_built_in_date_time_function_name(name: &str) -> bool {
 
 /// Definitions of errors raised in built-in functions module.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Built-in functions errors.
   #[derive(Debug, PartialEq)]
@@ -214,7 +214,7 @@ pub mod errors {
   }
 
   impl std::fmt::Display for BifError {
-    /// Implements [Display] trait for [BifError]s.
+    /// Implements [Display](std::fmt::Display) trait for [BifError]s.
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
       match self {
         BifError::UnknownFunctionName(name) => write!(f, "unknown built-in function name: {}", name),
