@@ -1,7 +1,7 @@
 /*
  * DMNTK - Decision Model and Notation Toolkit
  *
- * FEEL definitions.
+ * FEEL definitions
  *
  * Copyright 2018-2021 Dariusz Depta Engos Software <dariusz.depta@engos.software>
  *
@@ -20,12 +20,12 @@ use self::errors::*;
 use crate::names::{Name, QualifiedName};
 use crate::types::FeelType;
 use crate::values::Value;
-use dmntk_common::errors::DmntkError;
-use dmntk_common::feelify::Feelify;
-use dmntk_common::jsonify::Jsonify;
-use std::collections::{BTreeMap, HashSet};
-use std::convert::TryFrom;
-use std::ops::Deref;
+use dmntk_common::{DmntkError, Feelify, Jsonify, Stringify};
+use std::{
+  collections::{BTreeMap, HashSet},
+  convert::TryFrom,
+  ops::Deref,
+};
 
 /// Type alias for context entries.
 type FeelContextEntries = BTreeMap<Name, Value>;
@@ -35,7 +35,7 @@ type FeelContextEntries = BTreeMap<Name, Value>;
 pub struct FeelContext(FeelContextEntries);
 
 impl Default for FeelContext {
-  /// Creates a default empty [Context].
+  /// Creates a default empty [FeelContext].
   fn default() -> Self {
     Self(FeelContextEntries::new())
   }
@@ -50,7 +50,7 @@ impl Deref for FeelContext {
 
 impl TryFrom<Value> for FeelContext {
   type Error = DmntkError;
-  /// Tries to convert a [Value] to its [Context] representation.
+  /// Tries to convert a [Value] to its [FeelContext] representation.
   fn try_from(value: Value) -> Result<Self, Self::Error> {
     if let Value::Context(ctx) = value {
       Ok(ctx)
@@ -61,14 +61,21 @@ impl TryFrom<Value> for FeelContext {
 }
 
 impl From<FeelContext> for Value {
-  /// Converts a [Context] to its [Value] representation.
+  /// Converts this [FeelContext] to its [Value] representation.
   fn from(ctx: FeelContext) -> Self {
     Value::Context(ctx)
   }
 }
 
+impl Stringify for FeelContext {
+  /// Converts this [FeelContext] into its `TEXT` representation.
+  fn stringify(&self) -> String {
+    self.feelify()
+  }
+}
+
 impl Feelify for FeelContext {
-  /// Converts this [Context] into equivalent `FEEL` representation.
+  /// Converts this [FeelContext] into its `FEEL` representation.
   fn feelify(&self) -> String {
     format!(
       "{{{}}}",
@@ -83,7 +90,7 @@ impl Feelify for FeelContext {
 }
 
 impl Jsonify for FeelContext {
-  /// Converts this [Context] into equivalent `JSON` representation.
+  /// Converts this [FeelContext] into its `JSON` representation.
   fn jsonify(&self) -> String {
     format!(
       "{{{}}}",
@@ -98,15 +105,15 @@ impl Jsonify for FeelContext {
 }
 
 impl FeelContext {
-  /// Checks if this [Context] contains an entry pointed by [Name].
+  /// Checks if this [FeelContext] contains an entry pointed by [Name].
   pub fn contains_entry(&self, name: &Name) -> bool {
     self.0.contains_key(&name)
   }
-  /// Checks if this [Context] contains an entry pointed by [QualifiedName].
+  /// Checks if this [FeelContext] contains an entry pointed by [QualifiedName](crate::names::QualifiedName).
   pub fn contains_entries(&self, qname: &QualifiedName) -> bool {
     self.contains_deep(qname.as_slice())
   }
-  /// Sets a single value for specified entry name in this [Context].
+  /// Sets a single value for specified entry name in this [FeelContext].
   pub fn set_entry(&mut self, name: &Name, value: Value) {
     self.0.insert(name.clone(), value);
   }
@@ -132,13 +139,13 @@ impl FeelContext {
       self.0.insert(name.clone(), value.clone());
     }
   }
-  /// Creates an entry with a value for specified [QualifiedName].
+  /// Creates an entry with a value for specified [QualifiedName](crate::names::QualifiedName).
   /// All non existing intermediary contexts will be created.
   pub fn create_entry(&mut self, qname: &QualifiedName, value: Value) {
     self.create_deep(qname.as_slice(), value);
   }
-  /// Returns a list of flattened keys for this [Context].
-  pub fn flatten_keys(&self) -> Vec<String> {
+  /// Returns a list of flattened keys for this [FeelContext].
+  pub fn flatten_keys(&self) -> HashSet<String> {
     let mut keys: HashSet<String> = HashSet::new();
     for (key, value) in self.0.iter() {
       keys.insert(key.into());
@@ -147,18 +154,18 @@ impl FeelContext {
         if !sub_keys.is_empty() {
           for sub_key in sub_keys {
             keys.insert(sub_key.clone());
-            keys.insert(format!("{}.{}", key.to_string(), sub_key));
+            keys.insert(format!("{} . {}", key.to_string(), sub_key));
           }
         }
       }
       if let Value::List(items) = value {
-        for item in items.deref() {
+        for item in items.to_vec() {
           if let Value::Context(item_ctx) = item {
             let sub_keys = item_ctx.flatten_keys();
             if !sub_keys.is_empty() {
               for sub_key in sub_keys {
                 keys.insert(sub_key.clone());
-                keys.insert(format!("{}.{}", key.to_string(), sub_key));
+                keys.insert(format!("{} . {}", key.to_string(), sub_key)); //TODO add test for this case, nothing happened after adding spaces
               }
             }
           }
@@ -168,7 +175,7 @@ impl FeelContext {
         for name in a.keys() {
           let sub_key = name.to_string();
           keys.insert(sub_key.clone());
-          keys.insert(format!("{}.{}", key.to_string(), sub_key));
+          keys.insert(format!("{} . {}", key.to_string(), sub_key)); //TODO add test for this case, nothing happened after adding spaces
         }
       }
     }
@@ -238,9 +245,10 @@ impl FeelContext {
 
 /// Definitions of context errors.
 pub mod errors {
-  use crate::values::Value;
-  use dmntk_common::errors::DmntkError;
-  use dmntk_common::feelify::Feelify;
+  use {
+    crate::values::Value,
+    dmntk_common::{DmntkError, Feelify},
+  };
 
   /// Context errors.
   #[derive(Debug, PartialEq)]
@@ -275,15 +283,14 @@ mod tests {
   use crate::context::FeelContext;
   use crate::names::{Name, QualifiedName};
   use crate::values::Value;
-  use dmntk_common::feelify::Feelify;
-  use dmntk_common::jsonify::Jsonify;
+  use dmntk_common::Feelify;
+  use dmntk_common::Jsonify;
 
   #[test]
   fn test_context_default() {
     let ctx: FeelContext = Default::default();
     assert_eq!("{}", ctx.feelify());
-    let empty: Vec<String> = vec![];
-    assert_eq!(empty, ctx.flatten_keys());
+    assert!(ctx.flatten_keys().is_empty());
   }
 
   #[test]
@@ -331,7 +338,7 @@ mod tests {
     assert_eq!(false, ctx_a.contains_entries(&qname_a_b));
     assert_eq!(false, ctx_a.contains_entry(&name_a_b_c));
     assert_eq!(false, ctx_a.contains_entries(&qname_a_b_c));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"a".to_string()));
+    assert_eq!(true, ctx_a.flatten_keys().contains("a"));
     assert_eq!("10", ctx_a.get_entry(&name_a).unwrap().feelify().as_str());
   }
 
@@ -363,9 +370,9 @@ mod tests {
     assert_eq!(true, ctx_a.contains_entries(&qname_b));
     assert_eq!(true, ctx_a.contains_entries(&qname_b_married));
     assert_eq!(false, ctx_a.contains_entries(&qname_b_married_age));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"age".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b.married".to_string()));
+    assert_eq!(true, ctx_a.flatten_keys().contains("age"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b . married"));
     assert_eq!("49", ctx_a.get_entry(&name_age).unwrap().feelify().as_str());
     assert_eq!("{married:true}", ctx_a.get_entry(&name_b).unwrap().feelify().as_str());
   }
@@ -393,11 +400,11 @@ mod tests {
     assert_eq!(r#"{age:49,b:{c:{car:"opel"},married:true}}"#, ctx_a.feelify());
     assert_eq!(true, ctx_a.contains_entry(&name_age));
     assert_eq!(true, ctx_a.contains_entry(&name_b));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"age".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b.c".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b.married".to_string()));
-    assert_eq!(true, ctx_a.flatten_keys().contains(&"b.c.car".to_string()));
+    assert_eq!(true, ctx_a.flatten_keys().contains("age"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b . c"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b . married"));
+    assert_eq!(true, ctx_a.flatten_keys().contains("b . c . car"));
   }
 
   #[test]
@@ -505,12 +512,12 @@ mod tests {
     assert_eq!(r#"{a:1,b:2}"#, ctx.feelify());
     let keys = ctx.flatten_keys();
     assert_eq!(2, keys.len());
-    assert!(keys.contains(&"a".to_string()));
-    assert!(keys.contains(&"b".to_string()));
+    assert!(keys.contains("a"));
+    assert!(keys.contains("b"));
   }
 
   #[test]
-  fn flatten_two_levels() {
+  fn test_flatten_two_levels() {
     let name_a = Name::from("a");
     let name_b = Name::from("b");
     let name_c = Name::from("c");
@@ -524,16 +531,16 @@ mod tests {
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(6, keys.len());
-    assert!(keys.contains(&"a".to_string()));
-    assert!(keys.contains(&"b".to_string()));
-    assert!(keys.contains(&"c".to_string()));
-    assert!(keys.contains(&"c.a".to_string()));
-    assert!(keys.contains(&"c.b".to_string()));
-    assert!(keys.contains(&"c.c".to_string()));
+    assert!(keys.contains("a"));
+    assert!(keys.contains("b"));
+    assert!(keys.contains("c"));
+    assert!(keys.contains("c . a"));
+    assert!(keys.contains("c . b"));
+    assert!(keys.contains("c . c"));
   }
 
   #[test]
-  fn flatten_three_levels() {
+  fn test_flatten_three_levels() {
     let name_a = Name::from("a");
     let name_b = Name::from("b");
     let name_c = Name::from("c");
@@ -554,26 +561,26 @@ mod tests {
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(16, keys.len());
-    assert!(keys.contains(&"a".to_string()));
-    assert!(keys.contains(&"b".to_string()));
-    assert!(keys.contains(&"c".to_string()));
-    assert!(keys.contains(&"d".to_string()));
-    assert!(keys.contains(&"c.a".to_string()));
-    assert!(keys.contains(&"c.b".to_string()));
-    assert!(keys.contains(&"c.c".to_string()));
-    assert!(keys.contains(&"c.d".to_string()));
-    assert!(keys.contains(&"c.d.a".to_string()));
-    assert!(keys.contains(&"c.d.b".to_string()));
-    assert!(keys.contains(&"c.d.c".to_string()));
-    assert!(keys.contains(&"c.d.d".to_string()));
-    assert!(keys.contains(&"d.a".to_string()));
-    assert!(keys.contains(&"d.b".to_string()));
-    assert!(keys.contains(&"d.c".to_string()));
-    assert!(keys.contains(&"d.d".to_string()));
+    assert!(keys.contains("a"));
+    assert!(keys.contains("b"));
+    assert!(keys.contains("c"));
+    assert!(keys.contains("d"));
+    assert!(keys.contains("c . a"));
+    assert!(keys.contains("c . b"));
+    assert!(keys.contains("c . c"));
+    assert!(keys.contains("c . d"));
+    assert!(keys.contains("c . d . a"));
+    assert!(keys.contains("c . d . b"));
+    assert!(keys.contains("c . d . c"));
+    assert!(keys.contains("c . d . d"));
+    assert!(keys.contains("d . a"));
+    assert!(keys.contains("d . b"));
+    assert!(keys.contains("d . c"));
+    assert!(keys.contains("d . d"));
   }
 
   #[test]
-  fn flatten_names_with_additional_characters() {
+  fn test_flatten_names_with_additional_characters() {
     let name_a = Name::new(&["lorem", "ipsum", "dolor", "sit", "amet"]);
     let name_b = Name::new(&["b"]);
     let name_c = Name::new(&["now", "let", "'", "s", "go", "to", "the", "next", "paragraph"]);
@@ -586,10 +593,10 @@ mod tests {
     ctx_a.set_entry(&name_c, ctx_b.into());
     let keys = ctx_a.flatten_keys();
     assert_eq!(5, keys.len());
-    assert!(keys.contains(&"b".to_string()));
-    assert!(keys.contains(&"lorem ipsum dolor sit amet".to_string()));
-    assert!(keys.contains(&"now let ' s go to the next paragraph".to_string()));
-    assert!(keys.contains(&"now let ' s go to the next paragraph.Curabitur rhoncus + sodales odio in fringilla".to_string()));
-    assert!(keys.contains(&"Curabitur rhoncus + sodales odio in fringilla".to_string()));
+    assert!(keys.contains("b"));
+    assert!(keys.contains("lorem ipsum dolor sit amet"));
+    assert!(keys.contains("now let ' s go to the next paragraph"));
+    assert!(keys.contains("now let ' s go to the next paragraph . Curabitur rhoncus + sodales odio in fringilla"));
+    assert!(keys.contains("Curabitur rhoncus + sodales odio in fringilla"));
   }
 }
