@@ -14,7 +14,7 @@
 
 //! `FEEL` days and time durations.
 
-use {self::errors::*, dmntk_common::errors::DmntkError, regex::Regex, std::convert::TryFrom};
+use {self::errors::*, dmntk_common::DmntkError, regex::Regex, std::convert::TryFrom};
 
 /// Regular expression pattern for parsing days and time duration.
 const REGEX_DAYS_AND_TIME: &str =
@@ -228,7 +228,7 @@ impl TryFrom<&str> for FeelDaysAndTimeDuration {
 
 /// Definitions of date and time duration errors.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Date and time duration errors.
   #[derive(Debug, PartialEq)]
