@@ -18,7 +18,7 @@ use crate::temporal::date::{is_valid_date, FeelDate};
 use crate::temporal::ym_duration::FeelYearsAndMonthsDuration;
 use crate::temporal::zone::FeelZone;
 use chrono::{DateTime, Datelike, FixedOffset, Local, LocalResult, NaiveDate, NaiveDateTime, NaiveTime, TimeZone, Utc};
-use dmntk_common::errors::{DmntkError, Result};
+use dmntk_common::{DmntkError, Result};
 use regex::Regex;
 use std::cmp::Ordering;
 use std::convert::{TryFrom, TryInto};
