@@ -14,7 +14,7 @@
 
 /// Definitions of time errors.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Time errors.
   #[derive(Debug, PartialEq)]
