@@ -16,19 +16,23 @@
 
 //! Definitions of `FEEL` types.
 
-use crate::context::FeelContext;
-use crate::names::Name;
-use crate::values::{Value, Values, VALUE_NULL};
-use dmntk_common::errors::DmntkError;
-use std::ops::Deref;
-use std::{collections::BTreeMap, str::FromStr};
+use dmntk_common::Stringify;
+use {
+  crate::{
+    context::FeelContext,
+    names::Name,
+    values::{Value, Values, VALUE_NULL},
+  },
+  dmntk_common::DmntkError,
+  std::{collections::BTreeMap, str::FromStr},
+};
 
-pub type OptFeelType = Option<FeelType>;
+// pub type OptFeelType = Option<FeelType>;
 
-pub const FEEL_TYPE_ANY: FeelType = FeelType::Any;
-pub const FEEL_TYPE_NULL: FeelType = FeelType::Null;
+// pub const FEEL_TYPE_ANY: FeelType = FeelType::Any;
+// pub const FEEL_TYPE_NULL: FeelType = FeelType::Null;
 
-const FEEL_TYPE_NAME_ANY: &str = "Any";
+pub const FEEL_TYPE_NAME_ANY: &str = "Any";
 const FEEL_TYPE_NAME_BOOLEAN: &str = "boolean";
 const FEEL_TYPE_NAME_DATE: &str = "date";
 const FEEL_TYPE_NAME_DATE_AND_TIME: &str = "date and time";
@@ -71,6 +75,28 @@ pub enum FeelType {
   YearsAndMonthsDuration,
 }
 
+impl Stringify for FeelType {
+  fn stringify(&self) -> String {
+    match self {
+      FeelType::Any => FEEL_TYPE_NAME_ANY,
+      FeelType::Boolean => FEEL_TYPE_NAME_BOOLEAN,
+      FeelType::Context(_) => "context tbd",
+      FeelType::Date => FEEL_TYPE_NAME_DATE,
+      FeelType::DateTime => FEEL_TYPE_NAME_DATE_AND_TIME,
+      FeelType::DaysAndTimeDuration => FEEL_TYPE_NAME_DAYS_AND_TIME_DURATION,
+      FeelType::Function(_, _) => "function type (tbd)",
+      FeelType::List(_) => "list type (tbd)",
+      FeelType::Null => FEEL_TYPE_NAME_NULL,
+      FeelType::Number => FEEL_TYPE_NAME_NUMBER,
+      FeelType::Range(_) => "range type (tbd)",
+      FeelType::String => FEEL_TYPE_NAME_STRING,
+      FeelType::Time => FEEL_TYPE_NAME_TIME,
+      FeelType::YearsAndMonthsDuration => FEEL_TYPE_NAME_YEARS_AND_MONTHS_DURATION,
+    }
+    .to_string()
+  }
+}
+
 impl FromStr for FeelType {
   type Err = DmntkError;
   /// Converts a string to built-in type.
@@ -153,14 +179,14 @@ impl FeelType {
         }
       }
       FeelType::Function(_, _) => {
-        if let Value::FunctionDefinition(_, _, _) = value {
+        if let Value::FunctionDefinition(_, _) = value {
           return Some(value.clone());
         }
       }
       FeelType::List(typ) => {
         if let Value::List(items) = value {
           let mut result = vec![];
-          for item in items.deref() {
+          for item in items.to_vec() {
             if let Some(evaluated_value) = typ.derive(&item) {
               result.push(evaluated_value);
             }
@@ -369,7 +395,7 @@ impl FeelType {
 
 /// Definitions of errors raised in `types` module.
 pub mod errors {
-  use dmntk_common::errors::DmntkError;
+  use dmntk_common::DmntkError;
 
   /// Type errors.
   #[derive(Debug, PartialEq)]
@@ -394,7 +420,7 @@ pub mod errors {
     }
   }
 
-  /// Creates an instance of `UnknownTypeName` error.
+  /// Creates an [UnknownTypeName](TypeError::UnknownTypeName) error.
   pub fn unknown_type_name(name: &str) -> DmntkError {
     TypeError::UnknownTypeName(name.to_string()).into()
   }
